import React from 'react';
import validate from './validateInfo';
import useForm from './useFormSignup';
import { Link, useHistory} from 'react-router-dom';
import './form.css';


const FormSignup = ({ submitForm }) => {
  const history = useHistory();
  const { handleChange, handleSubmit, values, errors } = useForm(
    submitForm,
    validate
  );
  const sessionId = localStorage.getItem('sessionId');
  const sessionExpires = Number(localStorage.getItem('sessionExpires'));
  if (sessionId && Date.now() < sessionExpires) {
    history.push('/UserPage')
  }
  
  return (
    <div className='form-content-right'>
      <form onSubmit={handleSubmit} className='form' noValidate>
        <h1>
          Get started with us today! Create your account by filling out the
          information below.
        </h1>
        <div className='form-inputs'>
          <label className='form-label'>Full Name</label>
          <input
            className='form-input'
            type='text'
            name='username'
            placeholder='Enter your username'
            value={values.username}
            onChange={handleChange}
          />
          {errors.username && <p>{errors.username}</p>}
        </div>
        <div className='form-inputs'>
          <label className='form-label'>Email</label>
          <input
            className='form-input'
            type='email'
            name='email'
            placeholder='Enter your email'
            value={values.email}
            onChange={handleChange}
          />
          {errors.email && <p>{errors.email}</p>}
        </div>
        <div className='form-inputs'>
          <label className='form-label'>Password</label>
          <input
            className='form-input'
            type='password'
            name='password'
            placeholder='Enter your password'
            value={values.password}
            onChange={handleChange}
          />
          {errors.password && <p>{errors.password}</p>}
        </div>
        <div className='form-inputs'>
          <label className='form-label'>Confirm Password</label>
          <input
            className='form-input'
            type='password'
            name='password2'
            placeholder='Confirm your password'
            value={values.password2}
            onChange={handleChange}
          />
          {errors.password2 && <p>{errors.password2}</p>}
        </div>
        <button className='form-input-btn' type='submit'>
          Sign up
        </button>
        <span className='form-input-login'>
          Already have an account? Login <Link
          to='/login'> here </Link>
        </span>
      </form>
    </div>
  );
};

export default FormSignup;